@@ -12,13 +12,40 @@ import typing as t
 import zipfile
 from datetime import datetime
 
-__version__ = "0.2.4"
+__version__ = "0.2.5"
+
+
+class JMLHelpFormatter(argparse.HelpFormatter):
+    """Custom HelpFormatter to replace spaces with underscores in
+    metavars.
+    """
+    def __init__(self,
+                 prog,
+                 indent_increment=2,
+                 max_help_position=24,
+                 width=None):
+        super().__init__(prog, indent_increment=2, max_help_position=24, width=None)
+
+    def _metavar_formatter(self, action, default_metavar):
+        sformat = super()._metavar_formatter(action, default_metavar)
+
+        def new_format(tuple_size):
+            t = sformat(tuple_size)
+            return tuple(r.replace(" ", "_") if r else r for r in t)
+        return new_format
 
 
 parser = argparse.ArgumentParser(
-    prog="jml", description="Generiert aus einem Basisprojekt mehrere Projektversionen."
+    prog="jml", description="Generiert aus einem Basisprojekt mehrere Projektversionen.",
+    add_help=False,
+    formatter_class=JMLHelpFormatter
 )
 
+parser.add_argument(
+    "-h", "--help",
+    action="help",
+    help="Diesen Hilfetext anzeigen.",
+)
 parser.add_argument(
     "--version",
     action="version",
@@ -26,78 +53,65 @@ parser.add_argument(
     help="Programmversion anzeigen.",
 )
 
+# Argumente
 parser.add_argument("srcdir", metavar="IN", help="Pfad des Basisprojektes")
 parser.add_argument("outdir", metavar="OUT", help="Pfad des Zielordners")
+# Optionen
 parser.add_argument(
-    "-n",
-    "--name",
+    "-n", "--name",
     dest="name",
     action="store",
-    help="Name des Projektes. Standard ist der Name von IN",
+    help="Name der Projektversionen. Standard ist der Name von IN.",
 )
 parser.add_argument(
-    "--no-clear",
-    dest="clear",
-    action="store_false",
-    help="Löscht den Zielordner, bevor die Projektversionen erstellt werden",
-)
-parser.add_argument(
-    "-to",
-    "--tag-open",
-    dest="opening tag",
-    action="store",
-    help="Öffnender Tag für Aufgaben",
-)
-parser.add_argument(
-    "-tc",
-    "--tag-close",
-    dest="closing tag",
-    action="store",
-    help="Schließender Tag für Aufgaben",
-)
-parser.add_argument(
-    "-mlo",
-    "--ml-open",
-    dest="opening ml tag",
-    action="store",
-    help="Öffnender Tag für Musterlösungen",
-)
-parser.add_argument(
-    "-mlc",
-    "--ml-close",
-    dest="closing ml tag",
-    action="store",
-    help="Schließender Tag für Musterlösungen",
-)
-parser.add_argument(
-    "-mls",
-    "--ml-suffix",
-    dest="ml suffix",
-    action="store",
-    help="Suffix für die Musterlösung",
-)
-parser.add_argument(
-    "-nf",
-    "--name-format",
+    "-nf", "--name-format",
     dest="name format",
     action="store",
-    help="Format für die Namen der Projektversionen. Angabe als Python Formatstring. Kann die Variablen {project}, {version} und {date} enthalten.",
+    help="Format für die Namen der Projektversionen. Angabe als Python-Formatstring. Kann die Variablen {project}, {version} und {date} enthalten. Standard: {project}_{version}",
 )
 parser.add_argument(
-    "-i",
-    "--include",
+    "-mls", "--ml-suffix",
+    dest="ml suffix",
+    action="store",
+    help="Suffix für die Musterlösung. Standard: ML",
+)
+parser.add_argument(
+    "-to", "--tag-open",
+    dest="opening tag",
+    action="store",
+    help="Öffnende Aufgaben-Markierung.Standard: /*aufg*",
+)
+parser.add_argument(
+    "-tc", "--tag-close",
+    dest="closing tag",
+    action="store",
+    help="Schließende Aufgaben-Markierung. Standard: *aufg*/",
+)
+parser.add_argument(
+    "-mlo", "--ml-open",
+    dest="opening ml tag",
+    action="store",
+    help="Öffnende Lösungs-Markierung. Standard: //ml*",
+)
+parser.add_argument(
+    "-mlc", "--ml-close",
+    dest="closing ml tag",
+    action="store",
+    help="Schließende Lösungs-Markierung. Standard: //*ml",
+)
+parser.add_argument(
+    "-i", "--include",
     dest="include",
     action="extend",
     nargs="+",
-    help="Liste mit Dateiendungen, die nach JML-Kommentaren durchsucht werden sollen",
+    help="Liste mit Suchmustern für Dateien, die nach JML-Kommentaren durchsucht werden sollen. Standard: *.java",
 )
 parser.add_argument(
-    "-e",
-    "--exclude",
+    "-e", "--exclude",
     dest="exclude",
     action="extend",
     nargs="+",
-    help="Liste von Dateiendungen, die ignoriert werden",
+    help="Liste mit Suchmustern für Dateien, die komplett ignoriert werden und nicht in den Projektversionen auftauchen. Standard: *.class,*.ctxt,.DB_Store,Thumbs.db",
 )
 parser.add_argument(
     "-v",
@@ -105,26 +119,32 @@ parser.add_argument(
     dest="versions",
     action="extend",
     type=str,
-    help="Liste von Versionen, die erstellt werden sollen",
+    help="Liste von Versionen, die erstellt werden sollen.",
 )
 parser.add_argument(
     "--project-root",
     dest="project root",
     action="store",
-    help="if set to a prefix of IN, the folder structure in OUT will reflect the structure of the project root. For IN=/projects/foo/bar/my-project OUT=/projects/out project-root=/projects the resulting versions will be written to /projects/out/foo/bar",
+    help="Bestimmt ein Verzeichnis, das als Wurzelverzeichnis für mehrere Projekte dient. Wenn dies ein Prefix von IN ist, dann reflektiert die Ordnerstruktur in OUT die des Wurzelverzeichnisses. Für IN=/projects/foo/bar/my-project OUT=/projects/out project-root=/projects werden die Projektversionen dann in /projects/out/foo/bar erstellt.",
 )
 parser.add_argument(
     "--encoding",
     dest="encoding",
     action="store",
-    help="Codierung für Dateien. Standard: utf-8",
+    help="Zeichencodierung für Dateien. Standard: utf-8",
+)
+parser.add_argument(
+    "--no-clear",
+    dest="clear",
+    action="store_false",
+    help="Verhindert das Löschen der Projektversionen vor Ausführung.",
 )
 parser.add_argument(
     "--delete-empty",
     dest="keep empty files",
     action="store_false",
     default=True,
-    help="Als Standard werden leere Dateien in die Projektversionen kopiert. Diese Einstellung löscht leere Dateien.",
+    help="Leere Dateien aus den Projektversionen löschen.",
 )
 parser.add_argument(
     "-z",
@@ -132,14 +152,14 @@ parser.add_argument(
     dest="create zip",
     action="store_true",
     default=False,
-    help="Erstellt zusätzlich ZIP-Dateien zu den Projektversionen.",
+    help="Zusätzlich ZIP-Dateien zu den Projektversionen erstellen.",
 )
 parser.add_argument(
     "--no-ml",
     dest="delete ml",
     action="store_true",
     default=False,
-    help="Skip generating a ml version of the project. Note that the ml version is always generated, but this will delete the folder afterwards.",
+    help="Keinen Musterlösung erstellen. Beachte, dass die Musterlösung immer erstellt wird, um die zu erstellenden Projektversionen zu ermitteln. Diese Option löscht den Ordner am Ende wieder.",
 )
 
 parser.add_argument(
@@ -165,8 +185,10 @@ DEFAULT_CONFIG = {
     "name format": "{project}_{version}",
     "include": "*.java",
     "+include": "",
+    "-include": "",
     "exclude": "*.class,*.ctxt,.DS_Store,Thumbs.db,*.iml,.vscode,.eclipse",
     "+exclude": "",
+    "-exclude": "",
     "create zip": "no",
     "create zip only": "no",
     "create zip dir": "",
@@ -236,8 +258,8 @@ def main() -> None:
         interpolation=None, converters={"list": lambda v: re.split(r"[,;\n]+", v)}
     )
     # running list of patterns
-    excludes = set()
-    includes = set()
+    excludes = (set(), set())
+    includes = (set(), set())
     # set defaults
     config[CONFIG_SECTION] = DEFAULT_CONFIG
     settings = config[CONFIG_SECTION]
@@ -247,21 +269,27 @@ def main() -> None:
     if os.path.exists(user_config_file):
         config.read(user_config_file)
         debug(f"read config from user home at <{user_config_file}>")
-        excludes.update(settings.getlist("+exclude"))  # type: ignore
-        includes.update(settings.getlist("+include"))  # type: ignore
+        excludes[0].update(settings.getlist("+exclude"))  # type: ignore
+        excludes[1].update(settings.getlist("-exclude"))  # type: ignore
+        includes[0].update(settings.getlist("+include"))  # type: ignore
+        includes[1].update(settings.getlist("-include"))  # type: ignore
     # read project root config
     if project_root:
         root_config_file = os.path.join(project_root, CONFIG_FILE)
         if os.path.exists(root_config_file):
             config.read(root_config_file)
             debug(f"read config from project root at <{root_config_file}>")
-            excludes.update(settings.getlist("+exclude"))  # type: ignore
-            includes.update(settings.getlist("+include"))  # type: ignore
+        excludes[0].update(settings.getlist("+exclude"))  # type: ignore
+        excludes[1].update(settings.getlist("-exclude"))  # type: ignore
+        includes[0].update(settings.getlist("+include"))  # type: ignore
+        includes[1].update(settings.getlist("-include"))  # type: ignore
     # read project specific config
     if proj_config:
         config.read_dict(proj_config, source=proj_config_file)
-        excludes.update(settings.getlist("+exclude"))  # type: ignore
-        includes.update(settings.getlist("+include"))  # type: ignore
+        excludes[0].update(settings.getlist("+exclude"))  # type: ignore
+        excludes[1].update(settings.getlist("-exclude"))  # type: ignore
+        includes[0].update(settings.getlist("+include"))  # type: ignore
+        includes[1].update(settings.getlist("-include"))  # type: ignore
     # unset everything other than default keys
     for k in settings.keys():
         if k not in DEFAULT_CONFIG.keys():
@@ -279,12 +307,16 @@ def main() -> None:
         settings["name"] = os.path.basename(srcdir)
 
     # set include sets
-    excludes.update(settings.getlist("exclude"))  # type: ignore
-    includes.update(settings.getlist("include"))  # type: ignore
-    settings["exclude"] = ",".join(excludes)
-    settings["include"] = ",".join(includes)
+    excludes[0].update(settings.getlist("exclude"))  # type: ignore
+    includes[0].update(settings.getlist("include"))  # type: ignore
+    excludes[0].difference_update(excludes[1])  # type: ignore
+    includes[0].difference_update(includes[1])  # type: ignore
+    settings["exclude"] = ",".join(excludes[0])
+    settings["include"] = ",".join(includes[0])
     del settings["+exclude"]
     del settings["+include"]
+    del settings["-exclude"]
+    del settings["-include"]
 
     # show config for debugging
     if debug_flag:
